Class: Redis
- Inherits:
-
Object
- Object
- Redis
- Defined in:
- lib/redis/impending_cross_slot_error.rb,
lib/redis/key_hash/version.rb
Overview
Raised from Redis::KeyHash.all_in_one_slot! on a mismatch.
Generally, this indicates we were about to risk a Redis operation which is likely to produce a Redis error result like:
(error) CROSSSLOT Keys in request don't hash to the same slot
TODO: rubocop
TODO: rdoc
Defined Under Namespace
Modules: KeyHash Classes: ImpendingCrossSlotError